﻿using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace Infragistics.SampleGrid.Sources
{
    public class TestDataLoader : ITestDataLoader
    {
        private ObservableCollection<Category> _categories;
        private ObservableCollection<Product> _products;

        void GetCategories()
        {
            Stream stream =
                this.GetType().Assembly.GetManifestResourceStream("Infragistics.SampleGrid.Sources.Categories.xml");
            XDocument xml = XDocument.Load(System.Xml.XmlReader.Create(stream));

            this._categories = new ObservableCollection<Category>(
                (from category in xml.Descendants("Category")
                 select new Category
                 {
                     CategoryID = Convert.ToInt32(category.Element("CategoryID").Value),
                     CategoryName = category.Element("CategoryName").Value,
                     Description = category.Element("Description").Value,
                     Picture = new Binary
                     {
                         Bytes = (new UnicodeEncoding()).GetBytes(category.Element("Picture").Value)
                     },
                     Active = Convert.ToInt32(category.Element("Active").Value) == 1,
                     ActivationDate = Convert.ToDateTime(category.Element("ActivationDate").Value),
                     HomePage = category.Element("HomePage").Value
                 }).ToList());

            this.DataLoadingCompleted();
        }

        void GetProducts()
        {
            Stream stream =
                this.GetType().Assembly.GetManifestResourceStream("Infragistics.SampleGrid.Sources.Products.xml");
            XDocument xml = XDocument.Load(System.Xml.XmlReader.Create(stream));
            this._products = new ObservableCollection<Product>(
                (from category in xml.Descendants("Product")
                 select new Product
                 {
                     ProductID = Convert.ToInt32(category.Element("ProductID").Value),
                     ProductName = category.Element("ProductName").Value,
                     SupplierID = Convert.ToInt32(category.Element("SupplierID").Value),
                     CategoryID = Convert.ToInt32(category.Element("CategoryID").Value),
                     QuantityPerUnit = category.Element("QuantityPerUnit").Value,
                     UnitPrice = Convert.ToDecimal(category.Element("UnitPrice").Value, CultureInfo.InvariantCulture) * 1.0000m,
                     UnitsInStock = Convert.ToInt16(category.Element("UnitsInStock").Value),
                     UnitsOnOrder = Convert.ToInt16(category.Element("UnitsOnOrder").Value),
                     ReorderLevel = Convert.ToInt16(category.Element("ReorderLevel").Value),
                     Discontinued = Convert.ToInt32(category.Element("Discontinued").Value) == 1,
                     DiscontinuityDate = Convert.ToDateTime(category.Element("DiscontinuityDate").Value),
                     HomePage = category.Element("HomePage").Value
                 }).ToList());

            this.DataLoadingCompleted();
        }

        private void DataLoadingCompleted()
        {
            ObservableCollection<TestCategory> data;

            // process data only when both datas have been loaded
            if ((this._categories != null) && (this._products != null))
            {
                data = new ObservableCollection<TestCategory>();

                // proceeed only if there were no errors in loading the data
                // (this would be a problem if any of the data tables contains 0 records)
                if ((this._categories.Count > 0) && (this._products.Count > 0))
                {
                    (from c in this._categories
                     select new TestCategory()
                     {
                         ActivationDate = c.ActivationDate,
                         Active = c.Active,
                         CategoryID = c.CategoryID,
                         CategoryName = c.CategoryName,
                         Description = c.Description,
                         HomePage = c.HomePage,
                         Picture = c.Picture,
                         Products = c.Products
                     }).ToList().ForEach(x => data.Add(x));

                    foreach (TestCategory t in data)
                    {
                        t.CategoryProducts = new ObservableCollection<Product>();
                        (from p in this._products
                         where p.CategoryID == t.CategoryID
                         select p).ToList().ForEach(x => t.CategoryProducts.Add(x));
                    }
                }

                this.Data = data;

                if (this.Completed != null)
                    this.Completed(this, EventArgs.Empty);
            }
        }

        #region ITestDataLoader Members

        public event EventHandler Completed;

        public IEnumerable Data { get; private set; }

        public void BeginLoading()
        {
            this._categories = null;
            this._products = null;

            this.GetCategories();
            this.GetProducts();
        }

        #endregion
    }
}
