﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows.Media.Imaging;

namespace Infragistics.SampleGrid.Sources
{
    public interface ITestDataSource : IEnumerable
    {
        event EventHandler Completed;

        int? TargetItemsCount { get; set; }
        int? TargetDepth { get; set; }
        ITestDataLoader Loader { get; set; }
        bool IsLoading { get; set; }
        bool HasData { get; }

        ITestData CreateItem();
        IList GetList();
    }
    public interface ITestData
    {
        string StringData { get; set; }
        int IntegerData { get; set; }
        decimal DecimalData { get; set; }
        double DoubleData { get; set; }
        bool BoolData { get; set; }
        bool? BoolDataNull { get; set; }

        void ConvertFrom(object data);
    }
    public interface IComplexTestData
    {
        BitmapImage ImageData { get; set; }
        Uri UriData { get; set; }
        DateTime DateTimeData { get; set; }
        IEnumerable EnumerableData { get; set; }
        IEnumerable SecondaryEnumerableData { get; set; }

        event EventHandler<CollectionDetectedEventArgs> CollectionDetected;

        void ConvertFrom(object data);
        void ConvertFrom(object data, IEnumerable fillData);
    }
    public class CollectionDetectedEventArgs : EventArgs
    {
        public PropertyInfo TestProperty { get; set; }
        public IEnumerable Data { get; set; }
    }



    public class ObservableTestDataSource<T> : ObservableCollection<T>, ITestDataSource where T : ITestData
    {
        #region ITestDataSource Members

        public event EventHandler Completed;

        public bool? IsDataComplex { get; set; }

        public int? TargetItemsCount { get; set; }

        public int? TargetDepth { get; set; }

        public ITestDataLoader Loader
        {
            get { return this._loader; }
            set { this._loader = value; }
        }

        public bool IsLoading
        {
            get
            {
                return this._isLoading;
            }
            set
            {
                if ((this._isLoading == false) && (value == true))
                {
                    this._isLoading = value;

                    this._loader.Completed += Loader_Completed;
                    this._loader.BeginLoading();
                }
            }
        }

        public bool HasData
        {
            get
            {
                return this.GetList().Count > 0;
            }
        }

        #endregion // ITestDataSource Members

        #region Members

        private ITestDataLoader _loader;
        private IEnumerable _rawData;
        private int _depthCounter;
        private bool _isLoading;

        #endregion // Members

        #region Contructors

        public ObservableTestDataSource()
        {
            InitializeLoader();
        }

        public ObservableTestDataSource(bool isLoading)
            : this()
        {
            this.IsLoading = isLoading;
        }

        #endregion //Constructors

        #region Methods

        private void InitializeLoader()
        {
            if (this._loader == null)
                this._loader = new TestDataLoader();
        }

        void Loader_Completed(object sender, EventArgs e)
        {
            this._loader.Completed -= this.Loader_Completed;

            this._rawData = this._loader.Data;

            this.InitializeData();

            this._isLoading = false;

            if (Completed != null)
                Completed(this, new EventArgs());
        }

        private void InitializeData()
        {
            IEnumerator enumerator;
            object data;
            int count;
            T t;

            count = this.GetRawDataCount();

            if (count == 0)
                return;

            if (TargetItemsCount != null)
                count = (int)TargetItemsCount;

            enumerator = this._rawData.GetEnumerator();
            enumerator.Reset();

            for (int i = 0; i < count; i++)
            {
                if (!enumerator.MoveNext())
                {
                    enumerator.Reset();
                    enumerator.MoveNext();
                }

                data = enumerator.Current;

                this._depthCounter = 0;

                t = Activator.CreateInstance<T>();
                if (t is IComplexTestData)
                {
                    IComplexTestData newComplexData = t as IComplexTestData;

                    newComplexData.CollectionDetected += ComplexData_CollectionDetected;
                    newComplexData.ConvertFrom(data);
                    newComplexData.CollectionDetected -= ComplexData_CollectionDetected;
                }
                else
                    t.ConvertFrom(data);

                this.Add(t);
            }
        }

        private int GetRawDataCount()
        {
            int result = 0;

            if (this._rawData != null)
            {
                PropertyInfo info = this._rawData.GetType().GetProperty("Count", typeof(int));

                if (info != null)
                {
                    result = (int)info.GetValue(this._rawData, null);
                }
                else
                {
                    result = 0;

                    foreach (object tmp in this._rawData)
                        result++;
                }
            }

            return result;
        }

        void ComplexData_CollectionDetected(object sender, CollectionDetectedEventArgs e)
        {
            ObservableTestDataSource<T> collection;
            IEnumerable data;

            this._depthCounter++;
            if ((TargetDepth != null) && (this._depthCounter > TargetDepth))
            {
                this._depthCounter = 1;
                return;
            }

            collection = new ObservableTestDataSource<T>();
            e.TestProperty.SetValue(sender, collection, null);
            data = e.Data;
            foreach (object o in data)
            {
                T t = Activator.CreateInstance<T>();
                IComplexTestData newData = (t as IComplexTestData);

                newData.CollectionDetected += ComplexData_CollectionDetected;
                if (TargetDepth == null)
                    newData.ConvertFrom(o);
                else
                    newData.ConvertFrom(o, new object[] { o });
                newData.CollectionDetected -= ComplexData_CollectionDetected;

                collection.Add(t);
            }
        }

        public ITestData CreateItem()
        {
            return Activator.CreateInstance<T>();
        }

        public IList GetList()
        {
            return this;
        }

        #endregion // Methods
    }
    public class ObservableTestDataSource : ObservableTestDataSource<ComplexTestData>
    {
        public ObservableTestDataSource()
            : base()
        {
        }

        public ObservableTestDataSource(bool isLoading)
            : base(isLoading)
        {
        }
    }
    public class ObservableTestDataSourceNullableDateTime : ObservableTestDataSource<ComplexTestDataNullableDateTime>
    {
        public ObservableTestDataSourceNullableDateTime()
            : base()
        {
        }

        public ObservableTestDataSourceNullableDateTime(bool isLoading)
            : base(isLoading)
        {
        }
    }
    public class ObservableTestDataSourceCustomType : ObservableTestDataSource<ComplexTestDataCustomType>
    {
        public ObservableTestDataSourceCustomType()
            : base()
        {
        }

        public ObservableTestDataSourceCustomType(bool isLoading)
            : base(isLoading)
        {
        }
    }

    public class CustomType : INotifyPropertyChanged, IComparable, IConvertible
    {
        private int _customIntData;
        private string _customStringData;

        public CustomType()
            : this(0, null)
        {
        }

        public CustomType(int _customIntData, string _customStringData)
        {
            this._customIntData = _customIntData;
            this._customStringData = _customStringData;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public int Int
        {
            get
            {
                return this._customIntData;
            }

            set
            {
                this._customIntData = value;
                this.NotifyPropertyChanged("Int");
            }
        }

        public string String
        {
            get
            {
                return this._customStringData;
            }

            set
            {
                this._customStringData = value;
                this.NotifyPropertyChanged("String");
            }
        }

        public override string ToString()
        {
            return string.Format("CustomType.ToStr() {0}", this._customIntData);
        }

        #region INotifyPropertyChanged members

        private void NotifyPropertyChanged(string info)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }

        #endregion //INotifyPropertyChanged members

        #region IComparable Members

        public int CompareTo(object obj)
        {
            if (obj is CustomType)
            {
                return this._customIntData.CompareTo((obj as CustomType).Int);
            }
            else
            {
                throw new ArgumentException("Object is not of a type CustomType");
            }
        }

        #endregion

        #region IConvertible Members

        public TypeCode GetTypeCode()
        {
            return TypeCode.Object;
        }

        public bool ToBoolean(IFormatProvider provider)
        {
            return (this._customIntData > 0);
        }

        public byte ToByte(IFormatProvider provider)
        {
            return Convert.ToByte(this._customStringData);
        }

        public char ToChar(IFormatProvider provider)
        {
            return Convert.ToChar(this._customStringData);
        }

        public DateTime ToDateTime(IFormatProvider provider)
        {
            return DateTime.Today;
        }

        public decimal ToDecimal(IFormatProvider provider)
        {
            return Convert.ToDecimal(this._customIntData);
        }

        public double ToDouble(IFormatProvider provider)
        {
            return Convert.ToDouble(this._customIntData);
        }

        public short ToInt16(IFormatProvider provider)
        {
            return (Int16)this._customIntData;
        }

        public int ToInt32(IFormatProvider provider)
        {
            return this._customIntData;
        }

        public long ToInt64(IFormatProvider provider)
        {
            return Convert.ToInt64(this._customIntData);
        }

        public sbyte ToSByte(IFormatProvider provider)
        {
            return Convert.ToSByte(this._customStringData);
        }

        public float ToSingle(IFormatProvider provider)
        {
            return Convert.ToSingle(this._customIntData);
        }

        public string ToString(IFormatProvider provider)
        {
            return string.Format("CustomType.ToStr() {0}", this._customIntData);
        }

        public object ToType(Type conversionType, IFormatProvider provider)
        {
            return Convert.ChangeType(this._customIntData, conversionType, provider);
        }

        public ushort ToUInt16(IFormatProvider provider)
        {
            return Convert.ToUInt16(this._customIntData);
        }

        public uint ToUInt32(IFormatProvider provider)
        {
            return Convert.ToUInt32(this._customIntData);
        }

        public ulong ToUInt64(IFormatProvider provider)
        {
            return Convert.ToUInt64(this._customIntData);
        }

        #endregion
    }
    public class TestData : ITestData, INotifyPropertyChanged
    {
        #region ITestData Members

        public string StringData
        {
            get
            {
                return _stringData;
            }
            set
            {
                _stringData = value;
                OnPropertyChanged("StringData");
            }
        }

        public virtual int IntegerData
        {
            get
            {
                return _integerData;
            }
            set
            {
                _integerData = value;
                OnPropertyChanged("IntegerData");
            }
        }

        public decimal DecimalData
        {
            get
            {
                return _decimalData;
            }
            set
            {
                _decimalData = value;
                OnPropertyChanged("DecimalData");
            }
        }

        public double DoubleData
        {
            get
            {
                return _doubleData;
            }
            set
            {
                _doubleData = value;
                OnPropertyChanged("DoubleData");
            }
        }

        public bool BoolData
        {
            get
            {
                return _boolData;
            }
            set
            {
                _boolData = value;
                OnPropertyChanged("BoolData");
            }
        }

        public bool? BoolDataNull
        {
            get
            {
                return _boolDataNull;
            }
            set
            {
                _boolDataNull = value;
                OnPropertyChanged("BoolDataNull");
            }
        }

        public virtual void ConvertFrom(object data)
        {
            int index;
            List<PropertyInfo> dataPropertyInfos;

            if (data == null)
                return;

            dataPropertyInfos = new List<PropertyInfo>((data.GetType().GetProperties()));
            foreach (PropertyInfo thisPropertyInfo in typeof(ITestData).GetProperties())
            {
                index = 0;
                for (int i = dataPropertyInfos.Count; index < i; index++)
                {
                    PropertyInfo dataPropertyInfo = dataPropertyInfos[index];

                    if (thisPropertyInfo.PropertyType == dataPropertyInfo.PropertyType)
                    {
                        thisPropertyInfo.SetValue(this, dataPropertyInfo.GetValue(data, null), null);
                        break;
                    }
                }

                if (index < dataPropertyInfos.Count)
                    dataPropertyInfos.RemoveAt(index);
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Members

        private string _stringData;
        private int _integerData;
        private decimal _decimalData;
        private double _doubleData;
        private bool _boolData;
        private bool? _boolDataNull;

        #endregion

        #region Constructors

        public TestData()
        {
        }

        public TestData(object data)
        {
            ConvertFrom(data);
        }

        #endregion

        #region Methods

        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
    public class ComplexTestData : TestData, IComplexTestData
    {

        #region IComplexTestData Members

        public BitmapImage ImageData
        {
            get
            {
                return _imageData;
            }
            set
            {
                _imageData = value;
                OnPropertyChanged("ImageData");
            }
        }

        public Uri UriData
        {
            get
            {
                return _uriData;
            }
            set
            {
                _uriData = value;
                OnPropertyChanged("UriData");
            }
        }

        public DateTime DateTimeData
        {
            get
            {
                return _dateTimeData;
            }
            set
            {
                _dateTimeData = value;
                OnPropertyChanged("DateTimeData");
            }
        }

        public IEnumerable EnumerableData
        {
            get
            {
                return _enumerableData;
            }
            set
            {
                _enumerableData = value;
                OnPropertyChanged("EnumerableData");
            }
        }

        public IEnumerable SecondaryEnumerableData
        {
            get
            {
                return _secondaryEnumerableData;
            }
            set
            {
                _secondaryEnumerableData = value;
                OnPropertyChanged("SecondaryEnumerableData");
            }
        }

        public event EventHandler<CollectionDetectedEventArgs> CollectionDetected;

        #endregion

        #region Members

        private BitmapImage _imageData;
        private Uri _uriData;
        private IEnumerable _enumerableData;
        private IEnumerable _secondaryEnumerableData;
        private DateTime _dateTimeData;
        private bool _isCollectionDetected;

        #endregion

        #region Constructors

        public ComplexTestData()
        {
        }

        public ComplexTestData(object data)
        {
            ConvertFrom(data);
        }

        #endregion

        #region Methods

        public override void ConvertFrom(object data)
        {
            int index;
            List<PropertyInfo> dataPropertyInfos;

            if (data == null)
                return;

#if WINRT
            dataPropertyInfos = new List<PropertyInfo>((data.GetType().GetRuntimeProperties()));
            foreach (PropertyInfo thisPropertyInfo in typeof(ITestData).GetRuntimeProperties().Concat(typeof(IComplexTestData).GetRuntimeProperties()))
#else
            dataPropertyInfos = new List<PropertyInfo>((data.GetType().GetProperties()));
            foreach (PropertyInfo thisPropertyInfo in typeof(ITestData).GetProperties().Concat(typeof(IComplexTestData).GetProperties()))
#endif
            {
                index = 0;
                for (int i = dataPropertyInfos.Count; index < i; index++)
                {
                    PropertyInfo dataPropertyInfo = dataPropertyInfos[index];

                    if (thisPropertyInfo.PropertyType == dataPropertyInfo.PropertyType)
                    {
                        thisPropertyInfo.SetValue(this, dataPropertyInfo.GetValue(data, null), null);
                        break;
                    }
                    else if ((thisPropertyInfo.PropertyType == typeof(IEnumerable)) &&
                             (dataPropertyInfo.GetValue(data, null) is IEnumerable) &&
                             (!dataPropertyInfo.PropertyType.Equals(typeof(string))))
                    {
                        _isCollectionDetected = true;
                        CollectionDetected(this,
                                           new CollectionDetectedEventArgs()
                                           {
                                               TestProperty = thisPropertyInfo,
                                               Data = dataPropertyInfo.GetValue(data, null) as IEnumerable
                                           });
                        break;
                    }
                }

                if (index < dataPropertyInfos.Count)
                    dataPropertyInfos.RemoveAt(index);
            }
        }

        public void ConvertFrom(object data, IEnumerable fillData)
        {
            _isCollectionDetected = false;
            ConvertFrom(data);

            // give the chance to create desired number of hierarchies 
            if (_isCollectionDetected == false)
            {
                CollectionDetected(this, new CollectionDetectedEventArgs()
                {
#if WINRT
                    TestProperty = this.GetType().GetRuntimeProperty("EnumerableData"),
#else
                    TestProperty = this.GetType().GetProperty("EnumerableData"),
#endif
                    Data = fillData
                });
            }
        }

        #endregion
    }
    public class ComplexTestDataNullableDateTime : ComplexTestData
    {
        #region Public properties

        public DateTime? DateTimeDataNull
        {
            get
            {
                return _dateTimeDataNull;
            }
            set
            {
                _dateTimeDataNull = value;
                OnPropertyChanged("DateTimeDataNull");
            }
        }

        #endregion // Public properties

        #region Members

        private DateTime? _dateTimeDataNull;

        #endregion

        #region Constructors

        public ComplexTestDataNullableDateTime()
        {
        }

        public ComplexTestDataNullableDateTime(object data)
        {
            ConvertFrom(data);
        }

        #endregion //Constructors
    }

    public class ComplexTestDataCustomType : ComplexTestDataNullableDateTime
    {
        #region Public properties

        public CustomType CustomType
        {
            get
            {
                return _customType;
            }
            set
            {
                _customType = value;
                OnPropertyChanged("CustomType");
            }
        }

        public override int IntegerData
        {
            get
            {
                return base.IntegerData;
            }
            set
            {
                if (_customType != null)
                {
                    _customType.Int = value;
                }

                base.IntegerData = value;
            }
        }

        #endregion // Public properties

        #region Members

        private CustomType _customType;

        #endregion

        #region Constructors

        public ComplexTestDataCustomType()
        {
            this._customType = new CustomType();
            this._customType.Int = this.IntegerData;
            this._customType.String = this.StringData;
        }

        public ComplexTestDataCustomType(object data)
        {
            ConvertFrom(data);
            this._customType = new CustomType();
            this._customType.Int = this.IntegerData;
            this._customType.String = this.StringData;
        }

        #endregion //Constructors
    }


}
